@@ -45,7 +45,7 @@ namespace paperbox {
     using std::list;
     using std::map;
     using std::vector;
-    using boost::shared_ptr;
+    using std::tr1::shared_ptr;
 
     ///
 
@@ -254,6 +254,32 @@ namespace paperbox {
                 Gtk::Action::create("DocumentQuit", Gtk::Stock::QUIT),
                 sigc::mem_fun(*this, &MainWindow::on_menu_document_quit));
 
+        action_group_->add(Gtk::Action::create("ViewMenu", _("_View")));
+
+        action_group_->add(
+                Gtk::Action::create("ViewFirstPage",
+                    Gtk::Stock::GOTO_FIRST),
+                Gtk::AccelKey("<Alt>Up"),
+                sigc::mem_fun(*this, &MainWindow::on_menu_view_first_page));
+
+        action_group_->add(
+                Gtk::Action::create("ViewPreviousPage",
+                    Gtk::Stock::GO_BACK),
+                Gtk::AccelKey("<Alt>Left"),
+                sigc::mem_fun(*this, &MainWindow::on_menu_view_previous_page));
+        
+        action_group_->add(
+                Gtk::Action::create("ViewNextPage",
+                    Gtk::Stock::GO_FORWARD),
+                Gtk::AccelKey("<Alt>Right"),
+                sigc::mem_fun(*this, &MainWindow::on_menu_view_next_page));
+
+        action_group_->add(
+                Gtk::Action::create("ViewLastPage",
+                    Gtk::Stock::GOTO_LAST),
+                Gtk::AccelKey("<Alt>Down"),
+                sigc::mem_fun(*this, &MainWindow::on_menu_view_last_page));
+
         action_group_->add(Gtk::Action::create("HelpMenu", _("_Help")));
 
         action_group_->add(
@@ -276,6 +302,12 @@ namespace paperbox {
             "           <separator/>"
             "           <menuitem action='DocumentQuit'/>"
             "       </menu>"
+            "       <menu action='ViewMenu'>"
+            "           <menuitem action='ViewFirstPage'/>"
+            "           <menuitem action='ViewPreviousPage'/>"
+            "           <menuitem action='ViewNextPage'/>"
+            "           <menuitem action='ViewLastPage'/>"
+            "       </menu>"
             "       <menu action='HelpMenu'>"
             "           <menuitem action='HelpAbout'/>"
             "       </menu>"
@@ -381,6 +413,30 @@ namespace paperbox {
         hide();
     }
 
+    void
+    MainWindow::on_menu_view_first_page()
+    {
+        tile_view_->get_page_navigator().get_button_first().clicked();
+    }
+
+    void
+    MainWindow::on_menu_view_previous_page()
+    {
+        tile_view_->get_page_navigator().get_button_previous().clicked();
+    }
+
+    void
+    MainWindow::on_menu_view_next_page()
+    {
+        tile_view_->get_page_navigator().get_button_next().clicked();
+    }
+
+    void
+    MainWindow::on_menu_view_last_page()
+    {
+        tile_view_->get_page_navigator().get_button_last().clicked();
+    }
+
     void
     MainWindow::on_menu_about()
     {
@@ -393,7 +449,7 @@ namespace paperbox {
         list<Glib::ustring> authors;
         authors.push_back("Marko Anastasov <marko.anastasov@gmail.com>");
         dialog.set_authors(authors);
-        dialog.set_copyright("Copyright \xc2\xa9 2007-2008 Marko Anastasov");
+        dialog.set_copyright("Copyright \xc2\xa9 2007-2009 Marko Anastasov");
 
         dialog.set_website("http://live.gnome.org/PaperBox");
 
@@ -511,8 +567,11 @@ namespace paperbox {
         browser_->signal_retrieval_finished().connect(
             sigc::mem_fun(*this, &MainWindow::on_retrieval_finished));
 
-        browser_->signal_new_document().connect(
-            sigc::mem_fun(*this, &MainWindow::on_new_document));
+        browser_->signal_new_blank_documents().connect(
+            sigc::mem_fun(*this, &MainWindow::on_new_blank_documents));
+
+        browser_->signal_document_retrieved().connect(
+            sigc::mem_fun(*this, &MainWindow::on_document_retrieved));
 
         tile_view_->signal_tile_activated().connect(
             sigc::mem_fun(*this, &MainWindow::on_document_tile_selected));
@@ -541,28 +600,34 @@ namespace paperbox {
         show_widgets_after_retrieval();
     }
 
+    void
+    MainWindow::on_new_blank_documents(const vector<shared_ptr<Document> >& docs)
+    {
+        vector<shared_ptr<Document> >::const_iterator it(docs.begin());
+        vector<shared_ptr<Document> >::const_iterator end(docs.end());
+        for ( ; it != end; ++it) {
+            shared_ptr<DocumentTile> tile(new DocumentTile(thumbnailer_, *it));
+
+            bool ok = tiles_->add(tile);
+            if (! ok) return; // maybe not?
+
+            tile_view_->add_tile(*tile);
+        }
+    }
+
     // Invoked from Browser during idle time.
     // All this should be re-thought when we get xesam dbus api in tracker.
     void
-    MainWindow::on_new_document(const boost::shared_ptr<Document>& doc)
+    MainWindow::on_document_retrieved(const shared_ptr<Document>& doc)
     {
         shared_ptr<DocumentTile> tile(new DocumentTile(thumbnailer_, doc));
 
-        bool ok = tiles_->add(tile);
-        if (! ok) return; // maybe not?
-
-        tile_view_->add_tile(*tile);
-
         std::vector<Glib::ustring> tags = doc->get_tags();
         std::vector<Glib::ustring>::iterator it(tags.begin());
         std::vector<Glib::ustring>::iterator end(tags.end());
 
         for ( ; it != end; ++it)
             model_->add_tag(*it);
-
-        show_all_children();
-        if (in_retrieval_)
-            hide_widgets_in_retrieval();
     }
 
     void
